@@ -9,21 +9,21 @@
 // or else upgrading old project file will not work
 // reliably.
 // TODO QGIS 4: remove in favor of _QGIS_VERSION
-#define VERSION "3.16.0-Hannover"
-#define _QGIS_VERSION "3.16.0-Hannover"
+#define VERSION "3.18.0-Zürich"
+#define _QGIS_VERSION "3.18.0-Zürich"
 
 //used in vim src/core/qgis.cpp
 //The way below should work but it resolves to a number like 0110 which the compiler treats as octal I think
 //because debuggin it out shows the decimal number 72 which results in incorrect version status.
 //As a short term fix I (Tim) am defining the version in top level cmake. It would be good to
 //reinstate this more generic approach below at some point though
-//#define VERSION_INT 3160
+//#define VERSION_INT 3180
 // TODO QGIS 4: Remove in favor of _QGIS_VERSION_INT
-#define VERSION_INT 31600
-#define _QGIS_VERSION_INT 31600
-#define ABISYM(x) x ## 31600
+#define VERSION_INT 31800
+#define _QGIS_VERSION_INT 31800
+#define ABISYM(x) x ## 31800
 //used in main.cpp and anywhere else where the release name is needed
-#define RELEASE_NAME "Hannover"
+#define RELEASE_NAME "Zürich"
 
 #define QGIS_PLUGIN_SUBDIR "lib/qgis/plugins"
 #define QGIS_DATA_SUBDIR "share/qgis"
@@ -31,11 +31,11 @@
 #define QGIS_LIB_SUBDIR "lib"
 #define QGIS_QML_SUBDIR "qml"
 #define CMAKE_INSTALL_PREFIX "/usr"
-#define CMAKE_SOURCE_DIR "/build/qgis/src/qgis-3.16.0"
+#define CMAKE_SOURCE_DIR "/build/qgis/src/qgis-3.18.0"
 
 #define QGIS_SERVER_MODULE_SUBDIR "lib/qgis/server"
 
-#define QSCINTILLA_VERSION_STR "2.11.5"
+#define QSCINTILLA_VERSION_STR "2.12.0"
 
 #if defined( __APPLE__ )
 //used by Mac to find system or bundle resources relative to amount of bundling
@@ -54,6 +54,8 @@
 
 /* #undef HAVE_ORACLE */
 
+/* #undef HAVE_HANA */
+
 /* #undef SERVER_SKIP_ECW */
 
 #define HAVE_SERVER_PYTHON_PLUGINS
@@ -76,5 +78,15 @@
 
 /* #undef HAVE_STATIC_PROVIDERS */
 
+#define HAVE_EPT
+
+/* #undef HAVE_PDAL */
+#define PDAL_VERSION ""
+#define PDAL_VERSION_MAJOR_INT 
+#define PDAL_VERSION_MINOR_INT 
+#define PDAL_VERSION_MAJOR ""
+#define PDAL_VERSION_MINOR ""
+#define PDAL_VERSION_MICRO ""
+
 #endif
 
