@@ -45,7 +45,7 @@ class QgsRasterPyramid;
 
 /**
  * \ingroup core
- * A call back function for showing progress of gdal operations.
+ * \brief A call back function for showing progress of gdal operations.
  */
 int CPL_STDCALL progressCallback( double dfComplete,
                                   const char *pszMessage,
@@ -55,12 +55,12 @@ int CPL_STDCALL progressCallback( double dfComplete,
 class QgsCoordinateTransform;
 
 /**
-
-  \brief Data provider for GDAL layers.
-
-  This provider implements the interface defined in the QgsDataProvider class
-  to provide access to spatial data residing in a GDAL layers.
-
+ *
+ * \brief Data provider for GDAL layers.
+ *
+ * This provider implements the interface defined in the QgsDataProvider class
+ * to provide access to spatial data residing in a GDAL layers.
+ *
 */
 class QgsGdalProvider final: public QgsRasterDataProvider, QgsGdalProviderBase
 {
@@ -150,6 +150,7 @@ class QgsGdalProvider final: public QgsRasterDataProvider, QgsGdalProviderBase
 
     bool readBlock( int bandNo, int xBlock, int yBlock, void *data ) override;
     bool readBlock( int bandNo, QgsRectangle  const &viewExtent, int width, int height, void *data, QgsRasterBlockFeedback *feedback = nullptr ) override;
+
     double bandScale( int bandNo ) const override;
     double bandOffset( int bandNo ) const override;
     QList<QgsColorRampShader::ColorRampItem> colorTable( int bandNo )const override;
@@ -205,6 +206,11 @@ class QgsGdalProvider final: public QgsRasterDataProvider, QgsGdalProviderBase
 
     QgsPoint transformCoordinates( const QgsPoint &point, TransformType type ) override;
 
+    bool enableProviderResampling( bool enable ) override { mProviderResamplingEnabled = enable; return true; }
+    bool setZoomedInResamplingMethod( ResamplingMethod method ) override { mZoomedInResamplingMethod = method; return true; }
+    bool setZoomedOutResamplingMethod( ResamplingMethod method ) override { mZoomedOutResamplingMethod = method; return true; }
+    bool setMaxOversampling( double factor ) override { mMaxOversampling = factor; return true; }
+
   private:
     QgsGdalProvider( const QgsGdalProvider &other );
     QgsGdalProvider &operator=( const QgsGdalProvider & ) = delete;
@@ -344,6 +350,12 @@ class QgsGdalProvider final: public QgsRasterDataProvider, QgsGdalProviderBase
 
     //! Instance of GDAL transformer function used in transformCoordinates() for conversion between image and layer coordinates
     void *mGdalTransformerArg = nullptr;
+
+    bool canDoResampling(
+      int bandNo,
+      const QgsRectangle &reqExtent,
+      int bufferWidthPix,
+      int bufferHeightPix );
 };
 
 /**
@@ -353,9 +365,10 @@ class QgsGdalProviderMetadata final: public QgsProviderMetadata
 {
   public:
     QgsGdalProviderMetadata();
-    QVariantMap decodeUri( const QString &uri ) override;
-    QString encodeUri( const QVariantMap &parts ) override;
-    QgsGdalProvider *createProvider( const QString &uri, const QgsDataProvider::ProviderOptions &options ) override;
+    QVariantMap decodeUri( const QString &uri ) const override;
+    QString encodeUri( const QVariantMap &parts ) const override;
+    bool uriIsBlocklisted( const QString &uri ) const override;
+    QgsGdalProvider *createProvider( const QString &uri, const QgsDataProvider::ProviderOptions &options, QgsDataProvider::ReadFlags flags = QgsDataProvider::ReadFlags() ) override;
     QgsGdalProvider *createRasterDataProvider(
       const QString &uri,
       const QString &format,
