@@ -33,7 +33,7 @@ class QgsReadWriteContext;
 
 /**
  * \ingroup core
- * A base class for objects which belong to a layout.
+ * \brief A base class for objects which belong to a layout.
  * \since QGIS 3.0
  */
 class CORE_EXPORT QgsLayoutObject: public QObject, public QgsExpressionContextGenerator
@@ -163,6 +163,25 @@ class CORE_EXPORT QgsLayoutObject: public QObject, public QgsExpressionContextGe
       MapAtlasMargin, //!< Map atlas margin
       MapLayers, //!< Map layer set
       MapStylePreset, //!< Layer and style map theme
+      MapLabelMargin, //!< Map label margin
+      MapGridEnabled, //!< Map grid enabled
+      MapGridIntervalX, //!< Map grid interval X
+      MapGridIntervalY, //!< Map grid interval Y
+      MapGridOffsetX, //!< Map grid offset X
+      MapGridOffsetY, //!< Map grid offset Y
+      MapGridFrameSize, //!< Map grid frame size
+      MapGridFrameMargin, //!< Map grid frame margin
+      MapGridLabelDistance, //!< Map grid label distance
+      MapGridCrossSize, //!< Map grid cross size
+      MapGridFrameLineThickness, //!< Map grid frame line thickness
+      MapGridAnnotationDisplayLeft, //!< Map annotation display left
+      MapGridAnnotationDisplayRight, //!< Map annotation display right
+      MapGridAnnotationDisplayTop, //!< Map annotation display top
+      MapGridAnnotationDisplayBottom, //!< Map annotation display bottom
+      MapGridFrameDivisionsLeft, //!< Map frame division display left
+      MapGridFrameDivisionsRight, //!< Map frame division display right
+      MapGridFrameDivisionsTop, //!< Map frame division display top
+      MapGridFrameDivisionsBottom, //!< Map frame division display bottom
       //composer picture
       PictureSource, //!< Picture source url
       PictureSvgBackgroundColor, //!< SVG background color
@@ -174,12 +193,15 @@ class CORE_EXPORT QgsLayoutObject: public QObject, public QgsExpressionContextGe
       LegendTitle, //!< Legend title
       LegendColumnCount, //!< Legend column count
       //scalebar item
-      ScalebarFillColor, //!< Scalebar fill color
-      ScalebarFillColor2, //!< Scalebar secondary fill color
-      ScalebarLineColor, //!< Scalebar line color
-      ScalebarLineWidth, //!< Scalebar line width,
+      ScalebarFillColor, //!< Scalebar fill color (deprecated, use data defined properties on scalebar fill symbol 1 instead)
+      ScalebarFillColor2, //!< Scalebar secondary fill color (deprecated, use data defined properties on scalebar fill symbol 2 instead)
+      ScalebarLineColor, //!< Scalebar line color (deprecated, use data defined properties on scalebar line symbol instead)
+      ScalebarLineWidth, //!< Scalebar line width (deprecated, use data defined properties on scalebar line symbol instead)
       //table item
       AttributeTableSourceLayer, //!< Attribute table source layer
+      MapCrs, //!< Map CRS
+      StartDateTime, //!< Temporal range's start DateTime
+      EndDateTime, //!< Temporal range's end DateTime
     };
 
     /**
@@ -225,6 +247,7 @@ class CORE_EXPORT QgsLayoutObject: public QObject, public QgsExpressionContextGe
     /**
      * Returns a reference to the object's property collection, used for data defined overrides.
      * \see setDataDefinedProperties()
+     * \see DataDefinedProperty
      */
     const QgsPropertyCollection &dataDefinedProperties() const { return mDataDefinedProperties; } SIP_SKIP
 
@@ -232,6 +255,7 @@ class CORE_EXPORT QgsLayoutObject: public QObject, public QgsExpressionContextGe
      * Sets the objects's property collection, used for data defined overrides.
      * \param collection property collection. Existing properties will be replaced.
      * \see dataDefinedProperties()
+     * \see DataDefinedProperty
      */
     void setDataDefinedProperties( const QgsPropertyCollection &collection ) { mDataDefinedProperties = collection; }
 
@@ -301,7 +325,7 @@ class CORE_EXPORT QgsLayoutObject: public QObject, public QgsExpressionContextGe
      * \param parentElement is the parent DOM element to store the object's properties in
      * \param document DOM document
      * \param context read write context
-     * \returns true if write was successful
+     * \returns TRUE if write was successful
      * \see readObjectPropertiesFromElement()
      */
     bool writeObjectPropertiesToElement( QDomElement &parentElement, QDomDocument &document, const QgsReadWriteContext &context ) const;
@@ -311,7 +335,7 @@ class CORE_EXPORT QgsLayoutObject: public QObject, public QgsExpressionContextGe
      * \param parentElement is the parent DOM element for the object
      * \param document DOM document
      * \param context read write context
-     * \returns true if read was successful
+     * \returns TRUE if read was successful
      * \see writeObjectPropertiesToElement()
      */
     bool readObjectPropertiesFromElement( const QDomElement &parentElement, const QDomDocument &document, const QgsReadWriteContext &context );
