@@ -18,7 +18,7 @@
 
 #include "qgis_gui.h"
 #include "qgis_sip.h"
-#include "raster/qgsrasterlayer.h"
+#include "qgsrasterlayer.h"
 #include <QComboBox>
 
 class QgsMapLayer;
@@ -27,7 +27,7 @@ class QgsRasterDataProvider;
 /**
  * \class QgsRasterBandComboBox
  * \ingroup gui
- * A combobox widget which displays the bands present in a raster layer.
+ * \brief A combobox widget which displays the bands present in a raster layer.
  * \since QGIS 3.0
  */
 class GUI_EXPORT QgsRasterBandComboBox : public QComboBox
@@ -69,6 +69,11 @@ class GUI_EXPORT QgsRasterBandComboBox : public QComboBox
      */
     void setShowNotSetOption( bool show, const QString &string = QString() );
 
+    /**
+     * Returns a user-friendly band name for the specified \a band.
+     */
+    static QString displayBandName( QgsRasterDataProvider *provider, int band );
+
   public slots:
 
     /**
@@ -99,7 +104,6 @@ class GUI_EXPORT QgsRasterBandComboBox : public QComboBox
     QString mNotSetString;
     int mPrevBand = -1;
 
-    QString displayBandName( QgsRasterDataProvider *provider, int band ) const;
 
 
 };
