@@ -21,31 +21,21 @@ __author__ = 'Victor Olaya'
 __date__ = 'August 2012'
 __copyright__ = '(C) 2012, Victor Olaya'
 
-import os
 from pprint import pformat
 import time
 
-from qgis.PyQt.QtCore import QCoreApplication, Qt
-from qgis.PyQt.QtWidgets import QMessageBox, QPushButton, QSizePolicy, QDialogButtonBox
+from qgis.PyQt.QtCore import QCoreApplication
+from qgis.PyQt.QtWidgets import QMessageBox, QPushButton, QDialogButtonBox
 from qgis.PyQt.QtGui import QColor, QPalette
 
 from qgis.core import (Qgis,
-                       QgsProject,
                        QgsApplication,
-                       QgsProcessingUtils,
-                       QgsProcessingParameterDefinition,
                        QgsProcessingAlgRunnerTask,
                        QgsProcessingOutputHtml,
-                       QgsProcessingParameterVectorDestination,
-                       QgsProcessingOutputLayerDefinition,
-                       QgsProcessingParameterFeatureSink,
-                       QgsProcessingParameterRasterDestination,
                        QgsProcessingAlgorithm,
-                       QgsProcessingParameters,
                        QgsProxyProgressTask,
-                       QgsTaskManager)
+                       QgsProcessingFeatureSourceDefinition)
 from qgis.gui import (QgsGui,
-                      QgsMessageBar,
                       QgsProcessingAlgorithmDialogBase)
 from qgis.utils import iface
 
@@ -57,7 +47,6 @@ from processing.gui.BatchAlgorithmDialog import BatchAlgorithmDialog
 from processing.gui.AlgorithmDialogBase import AlgorithmDialogBase
 from processing.gui.AlgorithmExecutor import executeIterating, execute, execute_in_place
 from processing.gui.Postprocessing import handleAlgorithmResults
-from processing.gui.wrappers import WidgetWrapper
 
 from processing.tools import dataobjects
 
@@ -69,7 +58,7 @@ class AlgorithmDialog(QgsProcessingAlgorithmDialogBase):
 
         self.feedback_dialog = None
         self.in_place = in_place
-        self.active_layer = None
+        self.active_layer = iface.activeLayer() if self.in_place else None
 
         self.context = None
         self.feedback = None
@@ -80,18 +69,29 @@ class AlgorithmDialog(QgsProcessingAlgorithmDialogBase):
         if not self.in_place:
             self.runAsBatchButton = QPushButton(QCoreApplication.translate("AlgorithmDialog", "Run as Batch Process…"))
             self.runAsBatchButton.clicked.connect(self.runAsBatch)
-            self.buttonBox().addButton(self.runAsBatchButton, QDialogButtonBox.ResetRole) # reset role to ensure left alignment
+            self.buttonBox().addButton(self.runAsBatchButton,
+                                       QDialogButtonBox.ResetRole)  # reset role to ensure left alignment
         else:
-            self.active_layer = iface.activeLayer()
+            in_place_input_parameter_name = 'INPUT'
+            if hasattr(alg, 'inputParameterName'):
+                in_place_input_parameter_name = alg.inputParameterName()
+
+            self.mainWidget().setParameters({in_place_input_parameter_name: self.active_layer})
+
             self.runAsBatchButton = None
             has_selection = self.active_layer and (self.active_layer.selectedFeatureCount() > 0)
-            self.buttonBox().button(QDialogButtonBox.Ok).setText(QCoreApplication.translate("AlgorithmDialog", "Modify Selected Features")
-                                                                 if has_selection else QCoreApplication.translate("AlgorithmDialog", "Modify All Features"))
-            self.buttonBox().button(QDialogButtonBox.Close).setText(QCoreApplication.translate("AlgorithmDialog", "Cancel"))
+            self.buttonBox().button(QDialogButtonBox.Ok).setText(
+                QCoreApplication.translate("AlgorithmDialog", "Modify Selected Features")
+                if has_selection else QCoreApplication.translate("AlgorithmDialog", "Modify All Features"))
+            self.buttonBox().button(QDialogButtonBox.Close).setText(
+                QCoreApplication.translate("AlgorithmDialog", "Cancel"))
             self.setWindowTitle(self.windowTitle() + ' | ' + self.active_layer.name())
 
+        self.updateRunButtonVisibility()
+
     def getParametersPanel(self, alg, parent):
-        return ParametersPanel(parent, alg, self.in_place)
+        panel = ParametersPanel(parent, alg, self.in_place, self.active_layer)
+        return panel
 
     def runAsBatch(self):
         self.close()
@@ -99,73 +99,22 @@ class AlgorithmDialog(QgsProcessingAlgorithmDialogBase):
         dlg.show()
         dlg.exec_()
 
+    def resetAdditionalGui(self):
+        if not self.in_place:
+            self.runAsBatchButton.setEnabled(True)
+
+    def blockAdditionalControlsWhileRunning(self):
+        if not self.in_place:
+            self.runAsBatchButton.setEnabled(False)
+
     def setParameters(self, parameters):
         self.mainWidget().setParameters(parameters)
 
-    def getParameterValues(self):
-        parameters = {}
-
+    def createProcessingParameters(self, include_default=True):
         if self.mainWidget() is None:
-            return parameters
-
-        for param in self.algorithm().parameterDefinitions():
-            if param.flags() & QgsProcessingParameterDefinition.FlagHidden:
-                continue
-            if not param.isDestination():
-
-                if self.in_place and param.name() == 'INPUT':
-                    parameters[param.name()] = self.active_layer
-                    continue
-
-                try:
-                    wrapper = self.mainWidget().wrappers[param.name()]
-                except KeyError:
-                    continue
-
-                # For compatibility with 3.x API, we need to check whether the wrapper is
-                # the deprecated WidgetWrapper class. If not, it's the newer
-                # QgsAbstractProcessingParameterWidgetWrapper class
-                # TODO QGIS 4.0 - remove
-                if issubclass(wrapper.__class__, WidgetWrapper):
-                    widget = wrapper.widget
-                else:
-                    widget = wrapper.wrappedWidget()
-
-                if widget is None:
-                    continue
-
-                value = wrapper.parameterValue()
-                parameters[param.name()] = value
-
-                if not param.checkValueIsAcceptable(value):
-                    raise AlgorithmDialogBase.InvalidParameterValue(param, widget)
-            else:
-                if self.in_place and param.name() == 'OUTPUT':
-                    parameters[param.name()] = 'memory:'
-                    continue
-
-                dest_project = None
-                if not param.flags() & QgsProcessingParameterDefinition.FlagHidden and \
-                        isinstance(param, (QgsProcessingParameterRasterDestination,
-                                           QgsProcessingParameterFeatureSink,
-                                           QgsProcessingParameterVectorDestination)):
-                    if self.mainWidget().checkBoxes[param.name()].isChecked():
-                        dest_project = QgsProject.instance()
-
-                widget = self.mainWidget().outputWidgets[param.name()]
-                value = widget.getValue()
-
-                if value and isinstance(value, QgsProcessingOutputLayerDefinition):
-                    value.destinationProject = dest_project
-                if value:
-                    parameters[param.name()] = value
-                    if param.isDestination():
-                        context = dataobjects.createContext()
-                        ok, error = self.algorithm().provider().isSupportedOutputValue(value, param, context)
-                        if not ok:
-                            raise AlgorithmDialogBase.InvalidOutputExtension(widget, error)
-
-        return self.algorithm().preprocessParameters(parameters)
+            return {}
+        else:
+            return self.mainWidget().createProcessingParameters(include_default)
 
     def runAlgorithm(self):
         self.feedback = self.createFeedback()
@@ -173,7 +122,7 @@ class AlgorithmDialog(QgsProcessingAlgorithmDialogBase):
 
         checkCRS = ProcessingConfig.getSetting(ProcessingConfig.WARN_UNMATCHING_CRS)
         try:
-            parameters = self.getParameterValues()
+            parameters = self.createProcessingParameters()
 
             if checkCRS and not self.algorithm().validateInputCrs(parameters, self.context):
                 reply = QMessageBox.question(self, self.tr("Unmatching CRS's"),
@@ -189,28 +138,35 @@ class AlgorithmDialog(QgsProcessingAlgorithmDialogBase):
                 QMessageBox.warning(
                     self, self.tr('Unable to execute algorithm'), msg)
                 return
-            self.runButton().setEnabled(False)
+
+            self.blockControlsWhileRunning()
+            self.setExecutedAnyResult(True)
             self.cancelButton().setEnabled(False)
-            buttons = self.mainWidget().iterateButtons
+
             self.iterateParam = None
 
-            for i in range(len(list(buttons.values()))):
-                button = list(buttons.values())[i]
-                if button.isChecked():
-                    self.iterateParam = list(buttons.keys())[i]
+            for param in self.algorithm().parameterDefinitions():
+                if isinstance(parameters.get(param.name(), None), QgsProcessingFeatureSourceDefinition) and parameters[
+                        param.name()].flags & QgsProcessingFeatureSourceDefinition.FlagCreateIndividualOutputPerInputFeature:
+                    self.iterateParam = param.name()
                     break
 
             self.clearProgress()
             self.feedback.pushVersionInfo(self.algorithm().provider())
+            if self.algorithm().provider().warningMessage():
+                self.feedback.reportError(self.algorithm().provider().warningMessage())
+
             self.setProgressText(QCoreApplication.translate('AlgorithmDialog', 'Processing algorithm…'))
 
             self.setInfo(
-                QCoreApplication.translate('AlgorithmDialog', '<b>Algorithm \'{0}\' starting&hellip;</b>').format(self.algorithm().displayName()), escapeHtml=False)
+                QCoreApplication.translate('AlgorithmDialog', '<b>Algorithm \'{0}\' starting&hellip;</b>').format(
+                    self.algorithm().displayName()), escapeHtml=False)
 
             self.feedback.pushInfo(self.tr('Input parameters:'))
             display_params = []
             for k, v in parameters.items():
-                display_params.append("'" + k + "' : " + self.algorithm().parameterDefinition(k).valueAsPythonString(v, self.context))
+                display_params.append(
+                    "'" + k + "' : " + self.algorithm().parameterDefinition(k).valueAsPythonString(v, self.context))
             self.feedback.pushCommandInfo('{ ' + ', '.join(display_params) + ' }')
             self.feedback.pushInfo('')
             start_time = time.time()
@@ -241,9 +197,11 @@ class AlgorithmDialog(QgsProcessingAlgorithmDialogBase):
 
                 def on_complete(ok, results):
                     if ok:
-                        self.feedback.pushInfo(self.tr('Execution completed in {0:0.2f} seconds').format(time.time() - start_time))
+                        self.feedback.pushInfo(
+                            self.tr('Execution completed in {0:0.2f} seconds').format(time.time() - start_time))
                         self.feedback.pushInfo(self.tr('Results:'))
-                        self.feedback.pushCommandInfo(pformat(results))
+                        r = {k: v for k, v in results.items() if k not in ('CHILD_RESULTS', 'CHILD_INPUTS')}
+                        self.feedback.pushCommandInfo(pformat(r))
                     else:
                         self.feedback.reportError(
                             self.tr('Execution failed after {0:0.2f} seconds').format(time.time() - start_time))
@@ -256,10 +214,7 @@ class AlgorithmDialog(QgsProcessingAlgorithmDialogBase):
 
                     self.cancelButton().setEnabled(False)
 
-                    if not self.in_place:
-                        self.finish(ok, results, self.context, self.feedback)
-                    elif ok:
-                        self.close()
+                    self.finish(ok, results, self.context, self.feedback, in_place=self.in_place)
 
                     self.feedback = None
                     self.context = None
@@ -275,7 +230,9 @@ class AlgorithmDialog(QgsProcessingAlgorithmDialogBase):
                         task.executed.connect(on_complete)
                         self.setCurrentTask(task)
                 else:
-                    self.proxy_progress = QgsProxyProgressTask(QCoreApplication.translate("AlgorithmDialog", "Executing “{}”").format(self.algorithm().displayName()))
+                    self.proxy_progress = QgsProxyProgressTask(
+                        QCoreApplication.translate("AlgorithmDialog", "Executing “{}”").format(
+                            self.algorithm().displayName()))
                     QgsApplication.taskManager().addTask(self.proxy_progress)
                     self.feedback.progressChanged.connect(self.proxy_progress.setProxyProgress)
                     self.feedback_dialog = self.createProgressDialog()
@@ -298,8 +255,9 @@ class AlgorithmDialog(QgsProcessingAlgorithmDialogBase):
             except:
                 pass
             self.messageBar().clearWidgets()
-            self.messageBar().pushMessage("", self.tr("Wrong or missing parameter value: {0}").format(e.parameter.description()),
-                                          level=Qgis.Warning, duration=5)
+            self.messageBar().pushMessage("", self.tr("Wrong or missing parameter value: {0}").format(
+                e.parameter.description()),
+                level=Qgis.Warning, duration=5)
         except AlgorithmDialogBase.InvalidOutputExtension as e:
             try:
                 self.buttonBox().accepted.connect(lambda e=e:
@@ -313,15 +271,16 @@ class AlgorithmDialog(QgsProcessingAlgorithmDialogBase):
             self.messageBar().pushMessage("", e.message,
                                           level=Qgis.Warning, duration=5)
 
-    def finish(self, successful, result, context, feedback):
+    def finish(self, successful, result, context, feedback, in_place=False):
         keepOpen = not successful or ProcessingConfig.getSetting(ProcessingConfig.KEEP_DIALOG_OPEN)
 
-        if self.iterateParam is None:
+        if not in_place and self.iterateParam is None:
 
             # add html results to results dock
             for out in self.algorithm().outputDefinitions():
                 if isinstance(out, QgsProcessingOutputHtml) and out.name() in result and result[out.name()]:
-                    resultsList.addResult(icon=self.algorithm().icon(), name=out.description(), timestamp=time.localtime(),
+                    resultsList.addResult(icon=self.algorithm().icon(), name=out.description(),
+                                          timestamp=time.localtime(),
                                           result=result[out.name()])
             if not handleAlgorithmResults(self.algorithm(), context, feedback, not keepOpen, result):
                 self.resetGui()
@@ -330,8 +289,9 @@ class AlgorithmDialog(QgsProcessingAlgorithmDialogBase):
         self.setExecuted(True)
         self.setResults(result)
         self.setInfo(self.tr('Algorithm \'{0}\' finished').format(self.algorithm().displayName()), escapeHtml=False)
+        self.algorithmFinished.emit(successful, result)
 
-        if not keepOpen:
+        if not in_place and not keepOpen:
             self.close()
         else:
             self.resetGui()
